unit MemoryMappedFiles;

{$mode objfpc}{$H+}

interface

uses
  Windows, Math, Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls;

const
MMF_MaxBytes = 4080;
SingleSize = sizeof(Single);

NAN_Sleep     : single;
NAN_Reset     : single;   {reset Stepper destination to zero}
NAN_Run       : single;
NAN_Stop      : single;
NAN_Recognize : single;
NAN_Reconnect : single;
NAN_Calibrate : single;

type
  T_MMF_ByteArray= array[0..MMF_MaxBytes] of byte;
  T_MMF_ByteArrayPtr= ^T_MMF_ByteArray;

var
  MMF_Initialized: boolean = false;
  MMF_Filename: string;
  MMF_SharedByteArray: T_MMF_ByteArrayPtr;
  h_MMF_FileMapping: THandle;
  F_MMF_SharedByteArray: Pointer;

implementation

{$R *.lfm}

{ ================================== }
{       MMF - Private functions      }
{ ================================== }

procedure MMF_Initialize();
begin
  if h_MMF_FileMapping = 0 then
  begin

    { --------------------------------------------------------------------- }
    {  UNTESTED NAN INITIALIZATION  (byte order and syntax could be wrong)  }
    { --------------------------------------------------------------------- }
    x : array[1 .. 4] of byte;

    x := [0x01, 0x00, 0xC0, 0x7F]
    move(x, NAN_Sleep, SizeOf(single));

    x := [0x02, 0x00, 0xC0, 0x7F]
    move(x, NAN_Reset, SizeOf(single));

    x := [0x03, 0x00, 0xC0, 0x7F]
    move(x, NAN_Run, SizeOf(single));

    x := [0x04, 0x00, 0xC0, 0x7F]
    move(x, NAN_Stop, SizeOf(single));

    x := [0x05, 0x00, 0xC0, 0x7F]
    move(x, NAN_Recognize, SizeOf(single));

    x := [0x06, 0x00, 0xC0, 0x7F]
    move(x, NAN_Reconnect, SizeOf(single));

    x := [0x07, 0x00, 0xC0, 0x7F]
    move(x, NAN_Calibrate, SizeOf(single));
    { --------------------------------------------------------------------- }

    F_MMF_SharedByteArray := nil;
    MMF_Filename := 'Theremino1';
    h_MMF_FileMapping:=CreateFileMapping(INVALID_HANDLE_VALUE, nil, PAGE_READWRITE, 0, sizeof(T_MMF_ByteArray), PChar(MMF_Filename));
    { Get a pointer to memory mapped file memory block }
    if h_MMF_FileMapping <> 0 then
    begin
      F_MMF_SharedByteArray := MapViewOfFile(h_MMF_FileMapping, FILE_MAP_ALL_ACCESS, 0, 0, 0);
      if Assigned(F_MMF_SharedByteArray) then MMF_Initialized := true;
    end
  end
end;

procedure MMF_Finalize();
begin
  CloseHandle(h_MMF_FileMapping);
end;

procedure MMF_WriteSingle(Offset: Integer; Value: Single);
var
  pSingle: ^Single;
begin
  if MMF_Initialized then
    if Offset >= 0 then
      if Offset < MMF_MaxBytes then
      begin
        pSingle := @Value;
        CopyMemory(F_MMF_SharedByteArray + Offset, pSingle, SingleSize);
      end
end;

function MMF_ReadSingle(Offset: Integer): Single;
var
  pSingle: ^Single;
  Value: Single;
begin
  if MMF_Initialized then
    if Offset >= 0 then
      if Offset < MMF_MaxBytes then
      begin
        pSingle := @Value;
        CopyMemory(pSingle, F_MMF_SharedByteArray + Offset, SingleSize);
      end;
  result := Value;
end;

{ ================================== }
{ Theremino SLOTS - Public functions }
{ ================================== }

procedure MemoryMappedFile_Init();
begin
  MMF_Initialize();
end;

function ReadSlot(Slot: Integer): Single;
var
  Value: Single;
begin
  Value := MMF_ReadSingle(Slot * SingleSize);
  result := Value;
end;

function ReadSlot_NoNan(Slot: Integer): Single;
var
  Value: Single;
begin
  Value := MMF_ReadSingle(Slot * SingleSize);
  if IsNaN(Value) then Value := 0.0;
  result := Value;
end;

procedure WriteSlot(Slot: Integer; Value: Single);
begin
  MMF_WriteSingle(Slot * SingleSize, Value);
end;

procedure MemoryMappedFile_FillWithNanSleep();
var
  Slot: Integer;
begin
  For Slot := 0 to Trunc(MMF_MaxBytes / SingleSize) - 1 do
    MMF_WriteSingle(Slot * SingleSize, NAN_Sleep);
end;


end. {unit}


