Attribute VB_Name = "Module_MemoryMappedFiles"
Option Explicit

Private Declare Function CreateFileMapping Lib "kernel32.dll" Alias "CreateFileMappingA" (ByVal hFile As Long, _
                                         ByVal lpAttributes As Long, ByVal flProtect As Long, _
                                         ByVal dwMaximumSizeHigh As Long, ByVal dwMaximumSizeLow As Long, _
                                         ByVal lpName As String) As Long

Private Declare Function MapViewOfFile Lib "kernel32.dll" (ByVal hFile As Long, ByVal dwDesiredAccess As Long, _
                                         ByVal dwFileOffsetHigh As Long, ByVal dwFileOffsetLow As Long, _
                                         ByVal dwNumberOfBytesToMap As Long) As Long
                     
Private Declare Function UnmapViewOfFile Lib "kernel32.dll" (ByVal lpBaseAddress As Long) As Boolean

Private Declare Function CloseHandle Lib "kernel32" (ByVal hObject As Long) As Long

' Src -> *(Dst)
Private Declare Sub CopyMemoryWrite Lib "kernel32" Alias "RtlMoveMemory" (ByVal Dst As Long, _
                                                                          pSrc As Any, _
                                                                          ByVal ByteLen As Long)
' *(Src) -> Dst
Private Declare Sub CopyMemoryRead Lib "kernel32" Alias "RtlMoveMemory" (pDst As Any, _
                                                                         ByVal Src As Long, _
                                                                         ByVal ByteLen As Long)

Public Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (Destination As Any, _
                                                                    Source As Any, _
                                                                    ByVal Length As Long)


' ------------------------------------------------------- CONSTANTS
Private Const VT_BY_REF = &H4000&
Private Const FILE_SHARE_READ = &H1
Private Const FILE_SHARE_WRITE = &H2
Private Const MOVEFILE_REPLACE_EXISTING = &H1
Private Const FILE_ATTRIBUTE_TEMPORARY = &H100
Private Const FILE_ATTRIBUTE_NORMAL = &H80
Private Const FILE_BEGIN = 0
Private Const CREATE_NEW = 1
Private Const OPEN_EXISTING = 3
Private Const OPEN_ALWAYS = 4
Private Const GENERIC_READ = &H80000000
Private Const GENERIC_WRITE = &H40000000
Private Const PAGE_READWRITE = 4
Private Const FILE_MAP_WRITE = &H2
Private Const FILE_MAP_READ = &H4
Private Const FADF_FIXEDSIZE = &H10

' ======================================================================================
'  OPEN and CLOSE
' ======================================================================================
Private FileHandle As Long
Private MappingAddress As Long
Private FileLength As Long

Public Sub OpenMMF()
    Const Filename As String = "Theremino1"
    Const Length As Long = 4096
    ' --------------------------------------------------------- MAP THE FILE
    FileHandle = CreateFileMapping(0, _
                                 0, _
                                 PAGE_READWRITE, _
                                 0, _
                                 Length, _
                                 Filename)
    ' ---------------------------------------------------------
    If FileHandle = 0 Then CloseMMF: Exit Sub
    ' --------------------------------------------------------- MAP VIEW OF FILE
    MappingAddress = MapViewOfFile(FileHandle, _
                             FILE_MAP_WRITE, _
                             0, 0, 0)
    ' ---------------------------------------------------------
    If MappingAddress = 0 Then CloseMMF: Exit Sub
    ' ---------------------------------------------------------
    FileLength = Length
End Sub

Public Sub CloseMMF()
    If MappingAddress <> 0 Then UnmapViewOfFile MappingAddress
    If FileHandle <> 0 Then CloseHandle FileHandle
End Sub

' ======================================================================================
'  Write and Read - Single values
' ======================================================================================
Private Sub WriteSingle(ByVal Offset As Long, ByVal value As Single)
    If MappingAddress = 0 Then Exit Sub
    If Offset < 0 Or Offset >= FileLength Then Exit Sub
    Dim i As Long
    i = SingleToLong(value)
    CopyMemoryWrite MappingAddress + Offset, i, 4
End Sub
Private Function ReadSingle(ByVal Offset As Long) As Single
    If MappingAddress = 0 Then ReadSingle = 0: Exit Function
    If Offset < 0 Or Offset >= FileLength Then ReadSingle = 0: Exit Function
    Dim i As Long
    CopyMemoryRead i, MappingAddress + Offset, 4
    ReadSingle = LongToSingle(i)
End Function
Private Function ReadSingle_NoNan(ByVal Offset As Long) As Single
    If MappingAddress = 0 Then ReadSingle_NoNan = 0: Exit Function
    If Offset < 0 Or Offset >= FileLength Then ReadSingle_NoNan = 0: Exit Function
    Dim i As Long
    CopyMemoryRead i, MappingAddress + Offset, 4
    If (i > 3) And &H7F = &H7F Then
        ReadSingle_NoNan = LongToSingle(i)
    Else
        ReadSingle_NoNan = 0
    End If
End Function
Private Function LongToSingle(ByRef i As Long) As Single
    CopyMemory LongToSingle, i, 4
End Function
Private Function SingleToLong(ByRef i As Single) As Long
    CopyMemory SingleToLong, i, 4
End Function

' ======================================================================================
'  WriteSlot and ReadSlot
' ======================================================================================
Public Sub WriteSlot(ByVal slot As Long, ByVal value As Single)     ' time = 0.16 uS
    WriteSingle slot * 4, value
End Sub
Public Function ReadSlot(ByVal slot As Long) As Single              ' time = 0.09 uS
    ReadSlot = ReadSingle(slot * 4)
End Function
Public Function ReadSlot_NoNan(ByVal slot As Long) As Single        ' time = 0.15 uS
    ReadSlot_NoNan = ReadSingle_NoNan(slot * 4)
End Function


